-- yadafrac : yet another fractal exploration program, as a GtkAda training
--
-- Written by Thierry Bernier, see http://thiberlog.free.fr/
-- this source code is released to the public domain.
--
-- $Id: surface.adb,v 1.5 2005/02/25 18:08:16 Grands Exp $


with Ada.Text_IO; use Ada.Text_IO;

with Ada.Numerics.Elementary_Functions;
with Gdk.Types;             use Gdk.Types;
with Gdk.Drawable;          use Gdk.Drawable;

package body Surface is

  use Ada.Numerics.Elementary_Functions;

  function uint (X : Float) return Guint16 is
    Y : constant Float := 65536.0 * X;
  begin
    if    Y <=     0.0 then return 0;
    elsif Y >= 65535.0 then return 65535;
    else                    return Guint16(Y);
    end if;
  end uint;

  procedure Set_RGB (C : out Gdk_Color; X : in Float) is
    T       : constant Float := X * 2.0 * Ada.Numerics.PI;
    R, G, B : Float;
  begin
    R := (1.0 + Sin(6.0*T)) / 2.0;
    G := abs(Cos(8.0*T));
    B := abs(Sin(2.0*T));
    Set_RGB (C, uint(R), uint(G), uint(B));
  end Set_RGB;


--procedure Reset (S : out Surface_Type;
--                 D : in  Gdk_Drawable) is
--  Hue : Colormap_Type;
--begin
--  Reset (S, D, Hue);
--end;

  procedure Reset (S : out Surface_Type;
                   D : in  Gdk_Drawable)
  is
    Done : Boolean;
    Cmap : constant Gdk_Colormap := Get_Colormap(D);
    X    : Float;
  begin
    Gdk_New (S.GC, D);
    S.Drawable := D;
    for I in S.Colormap'First+1 .. S.Colormap'Last-1 loop
      X := (Float(I) - Float(S.Colormap'First)) / Float(S.Colormap'Length);
      Set_RGB (S.Colormap(I), X);
      Alloc_Color (Colormap => Cmap, Color => S.Colormap(I), Success => Done);
      S.Lists(I).Count := 0;
    end loop;
    Set_RGB (S.Colormap(S.Colormap'First), 0,0,0);
    Alloc_Color (Colormap => Cmap, Color => S.Colormap(S.Colormap'First),
                 Success  => Done);
    S.Lists(S.Colormap'First).Count := 0;
    Set_RGB (S.Colormap(S.Colormap'Last), 32767,32767,32767);
    Alloc_Color (Colormap => Cmap, Color => S.Colormap(S.Colormap'Last),
                 Success  => Done);
    S.Lists(S.Colormap'Last).Count := 0;
  end Reset;

  function Width  (S : Surface_Type) return Gint is
    W, H : Gint;
  begin
    Get_Size (S.Drawable, W, H);
    return W;
  end;

  function Height (S : Surface_Type) return Gint is
    W, H : Gint;
  begin
    Get_Size (S.Drawable, W, H);
    return H;
  end;

  procedure Draw_Point (S    : in out Surface_Type;
                        X, Y : in     Gint;
                        C    : in     Color_Type) is
  begin
    if S.Lists(C).Count >= S.Lists(C).Pool'Last then
      Set_Foreground (S.GC, S.Colormap(C));
      Draw_Points (S.Drawable, S.GC, S.Lists(C).Pool);
      S.Lists(C).Count := 0;
    end if;
    S.Lists(C).Count := S.Lists(C).Count + 1;
    S.Lists(C).Pool (S.Lists(C).Count) := (X, Y);
  exception
    when others => Put_Line (Color_Type'Image(C));
                   Put_Line (Natural'Image(S.Lists(C).Count));
                 raise;
  end Draw_Point;

  procedure Fill_Rect (S          : in out Surface_Type;
                       Xmin, Ymin : in     Gint;
                       Xmax, Ymax : in     Gint;
                       C          : in     Color_Type) is
  begin
    Set_Foreground (S.GC, S.Colormap(C));
    Draw_Rectangle (S.Drawable, S.GC, True, Xmin,        Ymin,
                                            Xmax-Xmin+1, Ymax-Ymin+1);
  end;

  procedure Flush (S : in out Surface_Type) is
  begin
    for C in S.Lists'Range loop
      if S.Lists(C).Count > 0 then
        Set_Foreground (S.GC, S.Colormap(C));
        Draw_Points (S.Drawable, S.GC, S.Lists(C).Pool(1..S.Lists(C).Count));
        S.Lists(C).Count := 0;
      end if;
    end loop;
  end Flush;

end Surface;


